@@ -3,7 +3,7 @@
 
 int main()
 {
-	KeySet *myConfig = ksNew(0);
+	KeySet *myConfig = ksNew(0, KS_END);
 	Key *key = keyNew("system/sw/MyApp",KEY_END);
 	KDB *handle = kdbOpen(key);
 
@@ -21,7 +21,7 @@ int main()
 	const char * key_name = keyName(result);
 	const char * key_value = keyString(result);
 	const char * key_comment = keyString(keyGetMeta(result, "comment"));
-	printf("key: %s value: %s comment: %s", key_name, key_value, key_comment);
+	printf("key: %s value: %s comment: %s\n", key_name, key_value, key_comment);
 
 	ksDel (myConfig); // delete the in-memory configuration
 
