@@ -49,18 +49,28 @@ void test_keynew()
 	succeed_if(key3.getName() == "system/sw/test", "key3 has wrong name");
 	succeed_if(key3.getBaseName() == "test", "wrong base name");
 	succeed_if (key3.getDirName() == "system/sw", "wrong dir name");
-	key3 = "system/other/name";
+	key3.setName("system/other/name");
 	succeed_if(key3.getName() == "system/other/name", "key3 has wrong name");
 	succeed_if(key3.getBaseName() == "name", "wrong base name");
 	succeed_if (key3.getDirName() == "system/other", "wrong dir name");
-	key3 += "base";
+	key3.addBaseName("base");
 	succeed_if(key3.getName() == "system/other/name/base", "key3 has wrong name");
 	succeed_if(key3.getBaseName() == "base", "wrong base name");
 	succeed_if (key3.getDirName() == "system/other/name", "wrong dir name");
-	key3 -= "name";
+	key3.setBaseName("name");
 	succeed_if(key3.getName() == "system/other/name/name", "key3 has wrong name");
 	succeed_if(key3.getBaseName() == "name", "wrong base name");
 	succeed_if (key3.getDirName() == "system/other/name", "wrong dir name");
+	key3.setName("system/name");
+	succeed_if(key3.getName() == "system/name", "key3 has wrong name");
+	succeed_if(key3.getBaseName() == "name", "wrong base name");
+	succeed_if (key3.getDirName() == "system", "wrong dir name");
+	/*
+	key3.addName("some/more");
+	succeed_if(key3.getName() == "system/name/some/more", "key3 has wrong name");
+	succeed_if(key3.getBaseName() == "more", "wrong base name");
+	succeed_if (key3.getDirName() == "system/name/some", "wrong dir name");
+	*/
 
 	// Key with name + value
 	Key key4 ("system/sw/test",
