@@ -45,44 +45,6 @@
 #include <kdbinternal.h>
 
 
-/**
- * Takes the first key and cuts off this common part
- * for all other keys.
- *
- * The first key is removed.
- *
- * Works only for system-configs.
- */
-static int renameBackendConfig(KeySet *config)
-{
-	Key *root;
-	Key *cur;
-	ssize_t systemSize = sizeof("system");
-	ssize_t rootSize = 0;
-
-	ksRewind(config);
-
-	root = ksNext (config);
-	rootSize = keyGetNameSize(root);
-	if (rootSize == -1) return -1;
-
-	keyDel (ksLookup (config, root, KDB_O_POP));
-
-	while ((cur = ksNext(config)) != 0)
-	{
-		ssize_t curSize = keyGetNameSize(cur);
-		if (curSize == -1) return -1;
-		for (ssize_t i=0; i<curSize-rootSize; ++i)
-		{
-			cur->key[i+systemSize] = cur->key[i+rootSize];
-		}
-		cur->keySize = curSize-rootSize+systemSize;
-	}
-
-	return 0;
-}
-
-
 /**Builds a backend out of the configuration supplied
  * from:
  *
@@ -123,7 +85,7 @@ Backend* elektraBackendOpen(KeySet *elektraConfig, KeySet *modules, Key *errorKe
 	KeySet *systemConfig = 0;
 	int failure = 0;
 
-	referencePlugins = ksNew(0);
+	referencePlugins = ksNew(0, KS_END);
 	ksRewind(elektraConfig);
 
 	root = ksNext (elektraConfig);
@@ -139,8 +101,8 @@ Backend* elektraBackendOpen(KeySet *elektraConfig, KeySet *modules, Key *errorKe
 			KeySet *cut = ksCut (elektraConfig, cur);
 			if (!strcmp(keyBaseName(cur), "config"))
 			{
-				systemConfig = cut;
-				renameBackendConfig (systemConfig);
+				systemConfig = elektraRenameKeys(cut, "system");
+				ksDel (cut);
 			}
 			else if (!strcmp(keyBaseName(cur), "getplugins"))
 			{
@@ -153,20 +115,16 @@ Backend* elektraBackendOpen(KeySet *elektraConfig, KeySet *modules, Key *errorKe
 			}
 			else if (!strcmp(keyBaseName(cur), "mountpoint"))
 			{
-				if (keyString(cur)[0] == '/')
-				{
-					backend->mountpoint = keyNew("",
-							KEY_VALUE, keyBaseName(root), KEY_END);
-					backend->mountpoint->key = elektraStrDup(keyString(cur));
-					backend->mountpoint->keySize = cur->dataSize;
-				} else {
-					backend->mountpoint = keyNew(keyString(cur),
-							KEY_VALUE, keyBaseName(root), KEY_END);
-				}
+				backend->mountpoint = keyNew("",
+						KEY_VALUE, keyBaseName(root), KEY_END);
+				elektraKeySetName(backend->mountpoint, keyString(cur),
+						KDB_O_CASCADING_NAME | KDB_O_EMPTY_NAME);
 
 				if (!backend->mountpoint)
 				{
-					ELEKTRA_ADD_WARNING(14, errorKey, keyValue(cur));
+					ELEKTRA_ADD_WARNINGF(14, errorKey,
+						"Could not create mountpoint with name %s and value %s",
+						keyString(cur), keyBaseName(root));
 					failure = 1;
 				}
 
