@@ -109,11 +109,11 @@ ksDel (myConfig); // delete keyset and all keys appended
  * with a list of keys. Either your first and only parameter is 0 or
  * your last parameter must be KEY_END.
  *
- * So, terminate with ksNew(0) or ksNew(20, ..., KS_END)
+ * So, terminate with ksNew(0, KS_END) or ksNew(20, ..., KS_END)
  *
  * For most uses
  * @code
-KeySet *keys = ksNew(0);
+KeySet *keys = ksNew(0, KS_END);
 // work with it
 ksDel (keys);
  * @endcode
@@ -156,10 +156,7 @@ ksDel (config);
  * The main benefit of taking a list of variant length parameters is to be able
  * to have one C-Statement for any possible KeySet.
  *
- * Due to ABI compatibility, the @p KeySet structure is only declared in kdb.h,
- * and not defined. So you can only declare @p pointers to @p KeySets in your
- * program.
- * See http://tldp.org/HOWTO/Program-Library-HOWTO/shared-libraries.html#AEN135
+ * @post the keyset is rewinded properly
  *
  * @see ksDel() to free the keyset afterwards
  * @see ksDup() to duplicate an existing keyset
@@ -184,6 +181,8 @@ KeySet *ksNew(size_t alloc, ...)
  *
  * @pre caller must call va_start and va_end
  * @par va the list of arguments
+ * @param alloc the allocation size
+ * @param va the list of variable arguments
  **/
 KeySet *ksVNew (size_t alloc, va_list va)
 {
@@ -209,9 +208,8 @@ KeySet *ksVNew (size_t alloc, va_list va)
 		return 0;
 	}
 	keyset->array[0] = 0;
-	
 
-	if (alloc != 1)
+	if (va && alloc != 1)
 	{
 		key = (struct _Key *) va_arg (va, struct _Key *);
 		while (key)
@@ -222,6 +220,8 @@ KeySet *ksVNew (size_t alloc, va_list va)
 
 	}
 
+	ksRewind(keyset);
+
 	return keyset;
 }
 
@@ -237,7 +237,7 @@ KeySet *ksVNew (size_t alloc, va_list va)
  * but there reference counter is updated, so both keysets
  * need ksDel().
  *
- * @param source has to be an initializised source KeySet
+ * @param source has to be an initialized source KeySet
  * @return a flat copy of source on success
  * @return 0 on NULL pointer
  * @see ksNew(), ksDel()
@@ -379,34 +379,122 @@ int ksClear(KeySet *ks)
 }
 
 
+/**
+ * @brief Compare by unescaped name only (not by owner, they are equal)
+ *
+ * Other Cmp* are based on this one.
+ *
+ * Is suitable for binary search (but may return wrong owner)
+ *
+ */
+static int keyCmpByName(const void *p1, const void *p2)
+{
+	Key *key1=*(Key **)p1;
+	Key *key2=*(Key **)p2;
+	const void *name1 = keyUnescapedName(key1);
+	const void *name2 = keyUnescapedName(key2);
+	size_t const nameSize1 = keyGetUnescapedNameSize(key1);
+	size_t const nameSize2 = keyGetUnescapedNameSize(key2);
+	int ret = 0;
+	if (nameSize1 == nameSize2)
+	{
+		ret = memcmp(name1, name2, nameSize2);
+	} else {
+		if (nameSize1 < nameSize2)
+		{
+			ret = memcmp(name1, name2, nameSize1);
+			if (ret==0) ret = -1;
+		} else {
+			ret = memcmp(name1, name2, nameSize2);
+			if (ret==0) ret = 1;
+		}
+	}
+	return ret;
+}
+
+static int keyCompareByOwner(const void *p1, const void *p2)
+{
+	Key *key1=*(Key **)p1;
+	Key *key2=*(Key **)p2;
+	const char *owner1 = keyValue(keyGetMeta(key1, "owner"));
+	const char *owner2 = keyValue(keyGetMeta(key2, "owner"));
+	if (!owner1 && !owner2) return 0;
+	if (!owner1) return -1;
+	if (!owner2) return 1;
+	return elektraStrCmp(owner1, owner2);
+}
 
 /**
  * @internal
  *
- * Used as a callback by the qsort() function
+ * Defines how keys are sorted in the keyset
+ *
+ * Compares by unescaped name, and if equal by owner
+ *
+ * Is suitable for binary search
+ *
+ * @see keyCmp, keyCmpByName
  */
-static int keyCmpInternal(const void *p1, const void *p2)
+static int keyCmpByNameOwner(const void *p1, const void *p2)
+{
+	int ret = keyCmpByName(p1, p2);
+
+	if (ret == 0)
+	{
+		return keyCompareByOwner(p1,p2);
+	}
+	return ret;
+}
+
+/**
+ * @brief Compare by name
+ *
+ * @return 
+ */
+static int keyCompareByName(const void *p1, const void *p2)
 {
 	Key *key1=*(Key **)p1;
 	Key *key2=*(Key **)p2;
 	const char *name1 = keyName(key1);
 	const char *name2 = keyName(key2);
-	int ret = strcmp(name1, name2);
 
-	/* sort by owner */
-	if (ret == 0)
-	{
-		const char *owner1 = keyOwner(key1);
-		const char *owner2 = keyOwner(key2);
-		if (!owner1 && !owner2) return 0;
-		if (!owner1) return -1;
-		if (!owner2) return 1;
-		return strcmp(owner1, owner2);
-	}
-
-	return ret;
+	return elektraStrCmp(name1, name2);
 }
 
+static int keyCompareByNameCase(const void *p1, const void *p2)
+{
+	Key *key1=*(Key **)p1;
+	Key *key2=*(Key **)p2;
+	const char *name1 = keyName(key1);
+	const char *name2 = keyName(key2);
+
+	return elektraStrCaseCmp(name1, name2);
+}
+
+static int keyCompareByNameOwner(const void *p1, const void *p2)
+{
+	int result = keyCompareByName(p1, p2);
+
+	if (result == 0)
+	{
+		return keyCompareByOwner(p1,p2);
+	}
+	return result;
+}
+
+
+static int keyCompareByNameOwnerCase(const void *p1, const void *p2)
+{
+	int result = keyCompareByNameCase(p1, p2);
+
+	if (result == 0)
+	{
+		return keyCompareByOwner(p1,p2);
+	}
+	return result;
+}
+
+
 
 /**
  * Compare the name of two keys.
@@ -423,9 +511,7 @@ static int keyCmpInternal(const void *p1, const void *p2)
  *
  * keyCmp() defines the sorting order for a KeySet.
  *
- * The following 3 points are the rules for null values.
- * They only take account when none of the preceding rules
- * matched.
+ * The following 3 points are the rules for null values:
  *
  * - A null pointer will be found to be smaller than every other
  * key. If both are null pointers, 0 is returned.
@@ -433,6 +519,8 @@ static int keyCmpInternal(const void *p1, const void *p2)
  * - A null name will be found to be smaller than every other
  * name. If both are null names, 0 is returned.
  *
+ * If the name is equal then:
+ *
  * - No owner will be found to be smaller then every other owner.
  * If both don't have a owner, 0 is returned.
  *
@@ -446,15 +534,9 @@ static int keyCmpInternal(const void *p1, const void *p2)
  * Given any Keys k1 and k2 constructed with keyNew(), following
  * equation hold true:
  *
- * @code
-// keyCmp(0,0) == 0
-// keyCmp(k1,0) ==  1
-// keyCmp(0,k2) == -1
- * @endcode
+ * @snippet testabi_rel cmp null
  *
- * You can write similar equation for the other rules.
- *
- * Here are some more examples with equation:
+ * Here are some more examples:
  * @code
 Key *k1 = keyNew("user/a", KEY_END);
 Key *k2 = keyNew("user/b", KEY_END);
@@ -463,6 +545,7 @@ Key *k2 = keyNew("user/b", KEY_END);
 // keyCmp(k2,k1) > 0
  * @endcode
  *
+ * And even more:
  * @code
 Key *k1 = keyNew("user/a", KEY_OWNER, "markus", KEY_END);
 Key *k2 = keyNew("user/a", KEY_OWNER, "max", KEY_END);
@@ -471,9 +554,8 @@ Key *k2 = keyNew("user/a", KEY_OWNER, "max", KEY_END);
 // keyCmp(k2,k1) > 0
  * @endcode
  *
- * @warning Do not try to strcmp the keyName() yourself because
- *      the used strcmp implementation is allowed to differ from
- *      simple ascii comparison.
+ * Do not strcmp the keyName() yourself because
+ * the result differs from simple ascii comparison.
  *
  * @param k1 the first key object to compare with
  * @param k2 the second key object to compare with
@@ -492,7 +574,7 @@ int keyCmp (const Key *k1, const Key *k2)
 	if (!k1->key) return -1;
 	if (!k2->key) return 1;
 
-	return keyCmpInternal(&k1, &k2);
+	return keyCmpByNameOwner(&k1, &k2);
 }
 
 /**
@@ -507,6 +589,9 @@ int keyCmp (const Key *k1, const Key *k2)
  *    metadata but the other has not, the key with the metadata
  *    is considered greater. If no key has metadata,
  *    they are considered to be equal.
+ *
+ * @param ka key to compare with
+ * @param kb other key to compare with
  */
 int elektraKeyCmpOrder(const Key *ka, const Key *kb)
 {
@@ -574,7 +659,7 @@ int ksNeedSync(const KeySet *ks)
  * @param ks the keyset object to work with
  * @return the number of keys that @p ks contains.
  * @return -1 on NULL pointer
- * @see ksNew(0), ksDel()
+ * @see ksNew(0, KS_END), ksDel()
  */
 ssize_t ksGetSize(const KeySet *ks)
 {
@@ -638,7 +723,7 @@ ssize_t ksSearchInternal(const KeySet *ks, const Key *toAppend)
 			break;
 		}
 		middle = left + ((right-left)/2);
-		cmpresult = keyCmpInternal(&toAppend, &ks->array[middle]);
+		cmpresult = keyCmpByNameOwner(&toAppend, &ks->array[middle]);
 		if (cmpresult > 0)
 		{
 			insertpos = left = middle + 1;
@@ -708,6 +793,8 @@ ssize_t ksAppendKey(KeySet *ks, Key *toAppend)
 		return -1;
 	}
 
+	keyLock(toAppend, KEY_LOCK_NAME);
+
 	result = ksSearchInternal(ks, toAppend);
 
 	if (result >= 0)
@@ -800,9 +887,6 @@ ssize_t ksAppend(KeySet *ks, const KeySet *toAppend)
 }
 
 
-static int keyCompareByNameOwner(const void *p1, const void *p2);
-
-
 /**
  * @internal
  *
@@ -870,13 +954,48 @@ ssize_t ksCopyInternal(KeySet *ks, size_t to, size_t from)
  *
  * Searches for the cutpoint inside the KeySet ks.
  * If found it cuts out everything which is below (see keyIsBelow()) this key.
- * If not found an empty keyset is returned.
+ * These keys will be missing in the keyset @p ks.
+ * Instead, they will be moved to the returned keyset.
+ * If @p cutpoint is not found an empty keyset is returned and @p ks
+ * is not changed.
  *
  * The cursor will stay at the same key as it was before.
- * If the cursor was inside the region of cutted (moved)
+ * If the cursor was inside the region of cut (moved)
  * keys, the cursor will be set to the key before
  * the cutpoint.
  *
+ * If you use ksCut() on a keyset you got from kdbGet() and plan to make
+ * a kdbSet() later, make sure that you keep all keys that should not
+ * be removed permanently. You have to keep the KeySet that was returned
+ * and the KeySet @p ks.
+ *
+ * @par Example:
+ *
+ * You have the keyset @p ks:
+ * - @p system/mountpoint/interest
+ * - @p system/mountpoint/interest/folder
+ * - @p system/mountpoint/interest/folder/key1
+ * - @p system/mountpoint/interest/folder/key2
+ * - @p system/mountpoint/other/key1
+ *
+ * When you use
+ * @snippet cut.c cut
+ *
+ * Then in @p returned are:
+ * - @p system/mountpoint/interest
+ * - @p system/mountpoint/interest/folder
+ * - @p system/mountpoint/interest/folder/key1
+ * - @p system/mountpoint/interest/folder/key2
+ *
+ * And in @p ks are:
+ * - @p system/mountpoint/other/key1
+ *
+ * So kdbSet() permanently removes all keys below
+ * @p system/mountpoint/interest.
+ *
+ * @see kdbGet() for explanation why you might get more keys than you
+ * requested.
+ *
  * @return a new allocated KeySet which needs to deleted with ksDel().
  *         The keyset consists of all keys (of the original keyset ks)
  *         below the cutpoint. If the key cutpoint exists, it will
@@ -906,7 +1025,7 @@ KeySet *ksCut(KeySet *ks, const Key *cutpoint)
 	}
 
 	// we found nothing
-	if (it == ks->size) return ksNew (0);
+	if (it == ks->size) return ksNew(0, KS_END);
 
 	// we found the cutpoint
 	found = it;
@@ -981,8 +1100,8 @@ KeySet *ksCut(KeySet *ks, const Key *cutpoint)
  * or keyDup().
  *
  *@code
-ks1=ksNew(0);
-ks2=ksNew(0);
+ks1=ksNew(0, KS_END);
+ks2=ksNew(0, KS_END);
 
 k1=keyNew("user/name", KEY_END); // ref counter 0
 ksAppendKey(ks1, k1); // ref counter 1
@@ -1052,17 +1171,17 @@ int elektraKsToMemArray(KeySet *ks, Key **buffer)
 
 	cursor_t cursor = ksGetCursor (ks);
 	ksRewind (ks);
-	size_t index = 0;
+	size_t idx = 0;
 
 	Key *key;
 	while ((key = ksNext (ks)) != 0)
 	{
-		buffer[index] = key;
-		++index;
+		buffer[idx] = key;
+		++idx;
 	}
 	ksSetCursor (ks, cursor);
 
-	return index;
+	return idx;
 }
 
 
@@ -1420,62 +1539,6 @@ int ksSetCursor(KeySet *ks, cursor_t cursor)
  *    Looking up Keys inside KeySets       *
  *******************************************/
 
-static int keyCompareByName(const void *p1, const void *p2)
-{
-	Key *key1=*(Key **)p1;
-	Key *key2=*(Key **)p2;
-	const char *name1 = keyName(key1);
-	const char *name2 = keyName(key2);
-
-	return strcmp(name1, name2);
-}
-
-static int keyCompareByNameCase(const void *p1, const void *p2)
-{
-	Key *key1=*(Key **)p1;
-	Key *key2=*(Key **)p2;
-	const char *name1 = keyName(key1);
-	const char *name2 = keyName(key2);
-
-	return elektraStrCaseCmp(name1, name2);
-}
-
-static int keyCompareByNameOwner(const void *p1, const void *p2)
-{
-	Key *key1=*(Key **)p1;
-	Key *key2=*(Key **)p2;
-	const char *name1 = keyName(key1);
-	const char *name2 = keyName(key2);
-	int result = strcmp(name1, name2);
-
-	if (result == 0)
-	{
-		const char *owner1 = keyOwner(key1);
-		const char *owner2 = keyOwner(key2);
-		return strcmp(owner1, owner2);
-	}
-	else return result;
-}
-
-
-static int keyCompareByNameOwnerCase(const void *p1, const void *p2)
-{
-	Key *key1=*(Key **)p1;
-	Key *key2=*(Key **)p2;
-	const char *name1 = keyName(key1);
-	const char *name2 = keyName(key2);
-	int result = elektraStrCaseCmp(name1, name2);
-
-	if (result == 0)
-	{
-		const char *owner1 = keyOwner(key1);
-		const char *owner2 = keyOwner(key2);
-		return elektraStrCaseCmp(owner1, owner2);
-	}
-	else return result;
-}
-
-
 /**
  * Look for a Key contained in @p ks that matches the name of the @p key.
  *
@@ -1581,11 +1644,7 @@ int f(KeySet *iterator, KeySet *lookup)
  */
 Key *ksLookup(KeySet *ks, Key * key, option_t options)
 {
-	Key *current;
-	Key ** found;
 	cursor_t cursor = 0;
-	size_t jump = 0;
-	/*If there is a known offset in the beginning jump could be set*/
 
 	if (!ks) return 0;
 
@@ -1593,8 +1652,13 @@ Key *ksLookup(KeySet *ks, Key * key, option_t options)
 
 	if (!key) return 0;
 
-	if (options & KDB_O_NOALL)
+	if ((options & KDB_O_NOALL)
+		// || (options & KDB_O_NOCASE)
+		// || (options & KDB_O_WITHOWNER)
+		) // binary search with nocase won't work
 	{
+		Key *current;
+		if (!(options & KDB_O_NOALL)) ksRewind(ks);
 		while ((current=ksNext(ks)) != 0)
 		{
 			if ((options & KDB_O_WITHOWNER) && (options & KDB_O_NOCASE))
@@ -1615,18 +1679,21 @@ Key *ksLookup(KeySet *ks, Key * key, option_t options)
 		if (current == 0) ksSetCursor (ks, cursor);
 		return current;
 	} else {
+		Key ** found;
+		size_t jump = 0;
+		/*If there is a known offset in the beginning jump could be set*/
 		if ((options & KDB_O_WITHOWNER) && (options & KDB_O_NOCASE))
 			found = (Key **) bsearch (&key, ks->array+jump, ks->size-jump,
 				sizeof (Key *), keyCompareByNameOwnerCase);
 		else if (options & KDB_O_WITHOWNER)
 			found = (Key **) bsearch (&key, ks->array+jump, ks->size-jump,
-				sizeof (Key *), keyCompareByNameOwner);
+				sizeof (Key *), keyCmpByNameOwner);
 		else if (options & KDB_O_NOCASE)
 			found = (Key **) bsearch (&key, ks->array+jump, ks->size-jump,
 				sizeof (Key *), keyCompareByNameCase);
 		else
-			found = (Key **) bsearch (&key, ks->array+jump, ks->size-jump,
-				sizeof (Key *), keyCompareByName);
+		found = (Key **) bsearch (&key, ks->array+jump, ks->size-jump,
+			sizeof (Key *), keyCmpByName);
 		if (options & KDB_O_DEL) keyDel (key);
 		if (found)
 		{
