@@ -33,40 +33,71 @@
 
 #include <stdlib.h>
 
+// #include <stdio.h>
+
+int elektraErrorOpen(Plugin *handle ELEKTRA_UNUSED, Key *parentKey)
+{
+	KeySet *conf = elektraPluginGetConfig(handle);
+
+	/*
+	FILE *f = fopen("error_plugin_debug.log", "a");
+	fprintf (f, "HUHU %s\n", keyName(parentKey));
+	ksRewind(conf);
+	while(ksNext(conf)) fprintf(f, "%s\n", keyName(ksCurrent(conf)));
+	fclose(f);
+	*/
+
+	if (ksLookupByName(conf, "/module", 0))
+	{
+		// suppress warnings + errors if it is just a module
+		return 0;
+	}
+
+	Key *warning = ksLookupByName(conf, "/on_open/warning", 0);
+	if (warning)
+	{
+		elektraTriggerWarnings (atoi(keyString(warning)), parentKey, "from error plugin in kdbOpen");
+	}
+
+	Key *error = ksLookupByName(conf, "/on_open/error", 0);
+	if (error)
+	{
+		if (parentKey)
+		{
+			elektraTriggerError (atoi(keyString(error)), parentKey, "from error plugin in kdbOpen");
+		}
+		return -1;
+	}
+	return 0;
+}
+
 int elektraErrorGet(Plugin *handle ELEKTRA_UNUSED, KeySet *returned, Key *parentKey ELEKTRA_UNUSED)
 {
-	KeySet *n;
-	ksAppend (returned, n = ksNew (30,
-		keyNew ("system/elektra/modules/error",
-			KEY_VALUE, "error plugin waits for your orders", KEY_END),
-		keyNew ("system/elektra/modules/error/exports", KEY_END),
-		keyNew ("system/elektra/modules/error/exports/get",
-			KEY_FUNC, elektraErrorGet,
-			KEY_END),
-		keyNew ("system/elektra/modules/error/exports/set",
-			KEY_FUNC, elektraErrorSet,
-			KEY_END),
-		keyNew ("system/elektra/modules/error/infos",
-			KEY_VALUE, "All information you want to know", KEY_END),
-		keyNew ("system/elektra/modules/error/infos/author",
-			KEY_VALUE, "Markus Raab <elektra@markus-raab.org>", KEY_END),
-		keyNew ("system/elektra/modules/error/infos/licence",
-			KEY_VALUE, "BSD", KEY_END),
-		keyNew ("system/elektra/modules/error/infos/description",
-			KEY_VALUE, "Validates key values using regular expressions", KEY_END),
-		keyNew ("system/elektra/modules/error/infos/provides",
-			KEY_VALUE, "error", KEY_END),
-		keyNew ("system/elektra/modules/error/infos/placements",
-			KEY_VALUE, "presetstorage", KEY_END),
-		keyNew ("system/elektra/modules/error/infos/needs",
-			KEY_VALUE, "", KEY_END),
-		keyNew ("system/elektra/modules/error/infos/version",
-			KEY_VALUE, PLUGINVERSION, KEY_END),
-		KS_END));
-	ksDel (n);
+	if (!strcmp (keyName(parentKey), "system/elektra/modules/error"))
+	{
+		KeySet *n;
+		ksAppend (returned, n = ksNew (30,
+			keyNew ("system/elektra/modules/error",
+				KEY_VALUE, "error plugin waits for your orders", KEY_END),
+			keyNew ("system/elektra/modules/error/exports", KEY_END),
+			keyNew ("system/elektra/modules/error/exports/open",
+				KEY_FUNC, elektraErrorOpen,
+				KEY_END),
+			keyNew ("system/elektra/modules/error/exports/get",
+				KEY_FUNC, elektraErrorGet,
+				KEY_END),
+			keyNew ("system/elektra/modules/error/exports/set",
+				KEY_FUNC, elektraErrorSet,
+				KEY_END),
+#include "readme_error.c"
+			keyNew ("system/elektra/modules/error/infos/version",
+				KEY_VALUE, PLUGINVERSION, KEY_END),
+			KS_END));
+		ksDel (n);
 
-	ksAppend (returned, n = elektraErrorSpecification());
-	ksDel (n);
+		ksAppend (returned, n = elektraErrorSpecification());
+		ksDel (n);
+	}
 	return 1;
 }
 
@@ -80,13 +111,13 @@ int elektraErrorSet(Plugin *handle ELEKTRA_UNUSED, KeySet *returned, Key *parent
 		meta = keyGetMeta (cur, "trigger/warnings");
 		if (meta)
 		{
-			elektraTriggerWarnings(atoi(keyString(meta)), parentKey, "from error plugin");
+			elektraTriggerWarnings(atoi(keyString(meta)), parentKey, "from error plugin in kdbSet");
 		}
 
 		meta = keyGetMeta (cur, "trigger/error");
 		if (meta)
 		{
-			elektraTriggerError (atoi(keyString(meta)), parentKey, "from error plugin");
+			elektraTriggerError (atoi(keyString(meta)), parentKey, "from error plugin in kdbSet");
 			return -1; /* error */
 		}
 	}
@@ -97,6 +128,7 @@ int elektraErrorSet(Plugin *handle ELEKTRA_UNUSED, KeySet *returned, Key *parent
 Plugin *ELEKTRA_PLUGIN_EXPORT(error)
 {
 	return elektraPluginExport("error",
+		ELEKTRA_PLUGIN_OPEN,	&elektraErrorOpen,
 		ELEKTRA_PLUGIN_GET,	&elektraErrorGet,
 		ELEKTRA_PLUGIN_SET,	&elektraErrorSet,
 		ELEKTRA_PLUGIN_END);
