@@ -97,22 +97,7 @@ int elektraFstabGet(Plugin *handle ELEKTRA_UNUSED, KeySet *returned, Key *parent
 			keyNew ("system/elektra/modules/fstab/exports/set",
 				KEY_FUNC, elektraFstabSet,
 				KEY_END),
-			keyNew ("system/elektra/modules/fstab/infos",
-				KEY_VALUE, "All information you want to know", KEY_END),
-			keyNew ("system/elektra/modules/fstab/infos/author",
-				KEY_VALUE, "Markus Raab <elektra@markus-raab.org>", KEY_END),
-			keyNew ("system/elektra/modules/fstab/infos/licence",
-				KEY_VALUE, "BSD", KEY_END),
-			keyNew ("system/elektra/modules/fstab/infos/description",
-				KEY_VALUE, "/Parses files in a syntax like /etc/fstab file", KEY_END),
-			keyNew ("system/elektra/modules/fstab/infos/provides",
-				KEY_VALUE, "storage", KEY_END),
-			keyNew ("system/elektra/modules/fstab/infos/placements",
-				KEY_VALUE, "getstorage setstorage", KEY_END),
-			keyNew ("system/elektra/modules/fstab/infos/needs",
-				KEY_VALUE, "", KEY_END),
-			keyNew ("system/elektra/modules/fstab/infos/recommends",
-				KEY_VALUE, "struct type path", KEY_END),
+#include "readme_fstab.c"
 			keyNew ("system/elektra/modules/fstab/infos/version",
 				KEY_VALUE, PLUGINVERSION, KEY_END),
 			keyNew ("system/elektra/modules/fstab/config/needs",
