@@ -58,20 +58,7 @@ int elektraSimpleiniGet(Plugin *handle ELEKTRA_UNUSED, KeySet *returned, Key *pa
 				KEY_FUNC, elektraSimpleiniGet, KEY_END),
 			keyNew ("system/elektra/modules/simpleini/exports/set",
 				KEY_FUNC, elektraSimpleiniSet, KEY_END),
-			keyNew ("system/elektra/modules/simpleini/infos",
-				KEY_VALUE, "All information you want to know", KEY_END),
-			keyNew ("system/elektra/modules/simpleini/infos/author",
-				KEY_VALUE, "Markus Raab <elektra@markus-raab.org>", KEY_END),
-			keyNew ("system/elektra/modules/simpleini/infos/licence",
-				KEY_VALUE, "BSD", KEY_END),
-			keyNew ("system/elektra/modules/simpleini/infos/description",
-				KEY_VALUE, "Very simple storage which writes out in a basic ini format", KEY_END),
-			keyNew ("system/elektra/modules/simpleini/infos/provides",
-				KEY_VALUE, "storage", KEY_END),
-			keyNew ("system/elektra/modules/simpleini/infos/placements",
-				KEY_VALUE, "getstorage setstorage", KEY_END),
-			keyNew ("system/elektra/modules/simpleini/infos/needs",
-				KEY_VALUE, "code null", KEY_END),
+#include "readme_simpleini.c"
 			keyNew ("system/elektra/modules/simpleini/infos/version",
 				KEY_VALUE, PLUGINVERSION, KEY_END),
 			keyNew ("system/elektra/modules/simpleini/config/needs",
