@@ -66,20 +66,7 @@ int elektraTclGet(Plugin *, KeySet *returned, Key *parentKey)
 				KEY_SIZE, sizeof (unserialise),
 				KEY_BINARY,
 				KEY_VALUE, &unserialise, KEY_END),
-			keyNew ("system/elektra/modules/tcl/infos",
-				KEY_VALUE, "All information you want to know", KEY_END),
-			keyNew ("system/elektra/modules/tcl/infos/author",
-				KEY_VALUE, "Markus Raab <elektra@markus-raab.org>", KEY_END),
-			keyNew ("system/elektra/modules/tcl/infos/licence",
-				KEY_VALUE, "BSD", KEY_END),
-			keyNew ("system/elektra/modules/tcl/infos/description",
-				KEY_VALUE, "Write out in tcl lists", KEY_END),
-			keyNew ("system/elektra/modules/tcl/infos/provides",
-				KEY_VALUE, "storage", KEY_END),
-			keyNew ("system/elektra/modules/tcl/infos/placements",
-				KEY_VALUE, "setstorage getstorage", KEY_END),
-			keyNew ("system/elektra/modules/tcl/infos/needs",
-				KEY_VALUE, "code", KEY_END),
+#include "readme_tcl.c"
 			keyNew ("system/elektra/modules/tcl/infos/version",
 				KEY_VALUE, PLUGINVERSION, KEY_END),
 			KS_END));
