@@ -27,6 +27,8 @@
 # include "kdbconfig.h"
 #endif
 
+#include <string.h>
+
 #include "template.h"
 
 int elektraTemplateOpen(Plugin *handle ELEKTRA_UNUSED, Key *errorKey ELEKTRA_UNUSED)
@@ -45,6 +47,31 @@ int elektraTemplateClose(Plugin *handle ELEKTRA_UNUSED, Key *errorKey ELEKTRA_UN
 
 int elektraTemplateGet(Plugin *handle ELEKTRA_UNUSED, KeySet *returned ELEKTRA_UNUSED, Key *parentKey ELEKTRA_UNUSED)
 {
+	if (!strcmp(keyName(parentKey), "system/elektra/modules/template"))
+	{
+		KeySet *contract = ksNew (30,
+		keyNew ("system/elektra/modules/template",
+			KEY_VALUE, "dbus plugin waits for your orders", KEY_END),
+		keyNew ("system/elektra/modules/template/exports", KEY_END),
+		keyNew ("system/elektra/modules/template/exports/open",
+			KEY_FUNC, elektraTemplateOpen, KEY_END),
+		keyNew ("system/elektra/modules/template/exports/close",
+			KEY_FUNC, elektraTemplateClose, KEY_END),
+		keyNew ("system/elektra/modules/template/exports/get",
+			KEY_FUNC, elektraTemplateGet, KEY_END),
+		keyNew ("system/elektra/modules/template/exports/set",
+			KEY_FUNC, elektraTemplateSet, KEY_END),
+		keyNew ("system/elektra/modules/template/exports/error",
+			KEY_FUNC, elektraTemplateError, KEY_END),
+#include ELEKTRA_README(template)
+		keyNew ("system/elektra/modules/template/infos/version",
+			KEY_VALUE, PLUGINVERSION, KEY_END),
+		KS_END);
+		ksAppend (returned, contract);
+		ksDel (contract);
+
+		return 1; /* success */
+	}
 	/* get all keys */
 
 	return 1; /* success */
