@@ -83,20 +83,7 @@ int elektraTracerGet(Plugin *handle, KeySet *returned, Key *parentKey)
 			keyNew ("system/elektra/modules/tracer/exports/error",
 				KEY_FUNC, elektraTracerError,
 				KEY_END),
-			keyNew ("system/elektra/modules/tracer/infos",
-				KEY_VALUE, "All information you want to know", KEY_END),
-			keyNew ("system/elektra/modules/tracer/infos/author",
-				KEY_VALUE, "Markus Raab <elektra@markus-raab.org>", KEY_END),
-			keyNew ("system/elektra/modules/tracer/infos/licence",
-				KEY_VALUE, "BSD", KEY_END),
-			keyNew ("system/elektra/modules/tracer/infos/description",
-				KEY_VALUE, "The first plugin", KEY_END),
-			keyNew ("system/elektra/modules/tracer/infos/provides",
-				KEY_VALUE, "logging", KEY_END),
-			keyNew ("system/elektra/modules/tracer/infos/needs",
-				KEY_VALUE, "", KEY_END),
-			keyNew ("system/elektra/modules/tracer/infos/placements",
-				KEY_VALUE, "pregetstorage postgetstorage presetstorage precommit postcommit prerollback postrollback", KEY_END),
+#include "readme_tracer.c"
 			keyNew ("system/elektra/modules/tracer/infos/version",
 				KEY_VALUE, PLUGINVERSION, KEY_END),
 			KS_END);
