@@ -56,44 +56,9 @@ int elektraUnameGet(Plugin *handle ELEKTRA_UNUSED, KeySet *returned, Key *parent
 			keyNew ("system/elektra/modules/uname/exports/set",
 				KEY_FUNC, elektraUnameSet,
 				KEY_END),
-			keyNew ("system/elektra/modules/uname/infos",
-				KEY_VALUE, "All information you want to know", KEY_END),
-			keyNew ("system/elektra/modules/uname/infos/author",
-				KEY_VALUE, "Markus Raab <elektra@markus-raab.org>", KEY_END),
-			keyNew ("system/elektra/modules/uname/infos/licence",
-				KEY_VALUE, "BSD", KEY_END),
-			keyNew ("system/elektra/modules/uname/infos/description",
-				KEY_VALUE,
-"Includes uname information into the key database.\n"
-"\n"
-"It defines following keynames below its mountpoint:\n"
-"\n"
-"sysname\n"
-"nodename\n"
-"release\n"
-"version\n"
-"machine\n"
-"\n"
-"To mount it, use src/kdb/kdb-full mount /tmp system/uname uname \n"
-"/tmp can be replaced with any existing file or directory.\n"
-"It will only be used for stat(), which is currently needed because\n"
-"plugins cannot be mounted without resolver.\n"
-"\n"
-"They are readonly.\n"
-				, KEY_END),
-			keyNew ("system/elektra/modules/uname/infos/provides",
-				KEY_VALUE, "storage", KEY_END),
-			keyNew ("system/elektra/modules/uname/infos/placements",
-				KEY_VALUE, "getstorage setstorage", KEY_END),
-			keyNew ("system/elektra/modules/uname/infos/needs",
-				KEY_VALUE, "", KEY_END),
-			keyNew ("system/elektra/modules/uname/infos/recommends",
-				KEY_VALUE, "struct type path", KEY_END),
+#include "readme_uname.c"
 			keyNew ("system/elektra/modules/uname/infos/version",
 				KEY_VALUE, PLUGINVERSION, KEY_END),
-			keyNew ("system/elektra/modules/uname/config/needs",
-				KEY_VALUE, "The configuration which is needed",
-				KEY_END),
 			KS_END);
 		ksAppend (returned, moduleConfig);
 		ksDel (moduleConfig);
@@ -109,6 +74,7 @@ int elektraUnameGet(Plugin *handle ELEKTRA_UNUSED, KeySet *returned, Key *parent
 
 	if (ret != 0)
 	{
+		// TODO: set error
 		return -1;
 	}
 
