@@ -29,8 +29,8 @@ Cmdline::Cmdline (int argc,
 	noNewline(),
 	test(),
 	recursive(),
+	resolver("resolver"),
 	strategy("preserve"),
-	overrideBase(),
 	verbose(),
 	version(),
 	withoutElektra(),
@@ -116,6 +116,17 @@ Cmdline::Cmdline (int argc,
 		long_options.push_back(o);
 		helpText += "-r --recursive           work in a recursive mode\n";
 	}
+	optionPos = acceptedOptions.find('R');
+	if (optionPos!=string::npos)
+	{
+		acceptedOptions.insert(optionPos+1, ":");
+		option o = {"resolver", required_argument, 0, 'R'};
+		long_options.push_back (o);
+		helpText +=
+				"-R --resolver <name>     the resolver plugin to use\n"
+				"                         if no resolver is given, the default resolver is used\n"
+				"";
+	}
 	optionPos = acceptedOptions.find('s');
 	if (optionPos!=string::npos)
 	{
@@ -123,27 +134,26 @@ Cmdline::Cmdline (int argc,
 		option o = {"strategy", required_argument, 0, 's'};
 		long_options.push_back(o);
 		helpText +=
-			"-s --strategy <name>     the strategy which should be used on conflicts\n"
-			"                       For merging:\n"
-			"                         preserve  .. fail on conflict (default)\n"
-			"                         ours      .. use ours for conflicts\n"
-			"                         theirs    .. use theirs for conflicts\n"
-			"                         base      .. use base for conflicts\n"
-			"                       Otherwise:\n"
-			"                         preserve  .. no old key is overwritten (default)\n"
-			"                         overwrite .. overwrite keys with same name\n"
-			"                         cut       .. completely cut at rootkey to make place for new keys\n"
+			"-s --strategy <name>     the strategy which should be used on conflicts.\n"
+			"                         To be precise, strategies handle deviations from the base\n"
+			"                         When and which strategies are used and what they do depends mostly on\n"
+			"                         the used base KeySet. For twoway-merge the base stays empty\n"
+			"                         Strategies can be chained. That means if one strategy\n"
+			"                         is not able to solve a conflict (i.e. conflicting deviations)\n"
+			"                         the next strategy in the chain is tried. This happens until the conflict\n"
+			"                         is solved or no strategies are left\n"
+			"                         For example, if you want to accept all deviations that do not conflict to theirs,\n"
+			"                         but use theirs in case of conflict use -s ourvalue,theirs\n"
+			"                         Currently the following strategies exist\n"
+			"                         preserve      .. no old key is overwritten (default)\n"
+			"                         ours          .. use always our version in case of conflict\n"
+			"                         theirs        .. use always their version in case of conflict\n"
+			"                         base          .. use always the base version in case of conflict\n"
+			"                         newkey        .. merge just those keys added by one side only\n"
+			"                         ourvalue      .. use our value if theirs is unmodified\n"
+			"                         theirvalue    .. use their value if ours is unmodified\n"
 			"";
 	}
-	if (acceptedOptions.find('b') != string::npos)
-	{
-		option b = {"overrideBase", no_argument, 0, 'b'};
-		long_options.push_back(b);
-		helpText += "-b --overrideBase        allow overriding the base with the merge result\n";
-		helpText += "                         with this option the merge command is no longer idempotent\n";
-		helpText += "                         (i.e. repeating the same command multiple times would yield\n";
-		helpText += "                         different results, because the base changes every time)\n";
-	}
 	if (acceptedOptions.find('v')!=string::npos)
 	{
 		option o = {"verbose", no_argument, 0, 'v'};
@@ -186,8 +196,8 @@ Cmdline::Cmdline (int argc,
 		case 'n': noNewline = true; break;
 		case 't': test = true; break;
 		case 'r': recursive = true; break;
+		case 'R': resolver = optarg; break;
 		case 's': strategy = optarg; break;
-		case 'b': overrideBase = true; break;
 		case 'v': verbose = true; break;
 		case 'V': version = true; break;
 		case 'E': withoutElektra= true; break;
