@@ -11,33 +11,7 @@
 #ifndef MOUNT_HPP
 #define MOUNT_HPP
 
-#include <command.hpp>
-#include <kdb.hpp>
-
-struct NameAlreadyInUseException : public CommandException
-{
-	virtual const char* what() const throw()
-	{
-		return "Name already used, will abort";
-	}
-};
-
-struct MountpointNotValid: public CommandException
-{
-	virtual const char* what() const throw()
-	{
-		return "The supplied name did not start with /\n"
-			"nor is it a valid keyname";
-	}
-};
-
-struct MountpointAlreadyInUseException : public CommandException
-{
-	virtual const char* what() const throw()
-	{
-		return "Mountpoint already used, will abort";
-	}
-};
+#include <mountbase.hpp>
 
 namespace kdb
 {
@@ -47,23 +21,14 @@ namespace tools
 }
 }
 
-class MountCommand : public Command
+class MountCommand : public MountBaseCommand
 {
-	void readMountConf();
 	void outputMtab();
 	void processArguments(Cmdline const& cl);
-	void fixRootKey(Cmdline const& cl);
-	void getName(Cmdline const& cl);
-	void getMountpoint(Cmdline const& cl);
 	void buildBackend(Cmdline const& cl);
 	void appendPlugins(Cmdline const& cl, kdb::tools::Backend & backend);
-	void askForConfirmation(Cmdline const& cl);
-	void doIt();
-
-	kdb::KeySet mountConf;
-	std::string name;
-	std::string path;
-	std::string mp;
+	bool readPluginConfig(Cmdline const& cl, size_t current_plugin);
+	void addConfig (std::string const& configBasePath, std::string const& name, std::string const& value);
 
 public:
 	MountCommand();
@@ -71,12 +36,12 @@ public:
 
 	virtual std::string getShortOptions()
 	{
-		return "id";
+		return "idR";
 	}
 
 	virtual std::string getSynopsis()
 	{
-		return "[path mountpoint] [plugin [..]]";
+		return "[path mountpoint] [plugin [config] [..]]";
 	}
 
 	virtual std::string getShortHelpText()
@@ -89,10 +54,25 @@ public:
 		return
 			"path .. a filename (absolute for system, relative for cascading or user)\n"
 			"mountpoint .. where to mount the backend, start with / for cascading mp\n"
-			"plugin .. a list of plugins to mount at that place\n"
+			"plugin .. a list of plugins and their config to mount at that place\n"
+			"Each plugin my be followed by a list of keys and corresponding values that will be\n"
+			"written below the backend config. For example param1=value1\n"
 			"\n"
 			"With the -i option, the mounting will be done interactively\n"
 			"With no options and no arguments, the current mountpoints will be listed\n"
+			"\n"
+			"Examples:\n"
+			"\n"
+			"kdb mount /etc/file system/file plugin1 plugin1config=config1 plugin2 plugin2config=config2\n"
+			"\n"
+//			"recode will be used both for renaming of value+name\n"
+//			"kdb mount path=s.ini,recode=utf8..latin1,name=backendname /path/to/mount\n"
+//			"          ni,name=pluginname rename,rebase=/path iconv\n"
+//			"\n"
+//			"mount openicc DB\n"
+//			"kdb mount --resolver openicc\n"
+//			"          OpenICC_device_config_DB.json /org/freedesktop/openicc\n"
+//			"          yajl rename,rebase=/org/freedesktop/openicc,recase=lower\n"
 			;
 	}
 
